set(CMAKE_MODULE_PATH "${CMAKE_CURRENT_LIST_DIR}/../../cmake-3.23.2-patch/Modules") # For patches.

set(CMAKE_SYSTEM_NAME Generic)

# You can set the tool path here in stead of setting the environment variable `Path` on Windows.
set(TOOLCHAIN_PATH C:/Renesas/CS+/CC/CC-RH/V2.04.00/bin) # Quote the path with "..." if it includes space.

set(CMAKE_C_COMPILER ${TOOLCHAIN_PATH}/ccrh.exe)

############################
macro(SET_DIRECTORY_OPTIONS)
############################

set(CMAKE_C_STANDARD 99)
#set(CMAKE_C_STANDARD_REQUIRED ON) # CMake's default is OFF.
#set(CMAKE_C_EXTENSIONS OFF) # CC-RX/RL/RH's default is ON and CC-RX has no strict standard option.

add_compile_options( # Somehow space-separated flags (i.e. flags list) cannot be used in the following generator expressions.
$<$<COMPILE_LANGUAGE:C>:-Xcpu=g3kh>
$<$<COMPILE_LANGUAGE:C>:-goptimize>
$<$<COMPILE_LANGUAGE:C>:-g>
$<$<COMPILE_LANGUAGE:C>:-g_line>
$<$<COMPILE_LANGUAGE:C>:-Xcharacter_set=utf8>
$<$<COMPILE_LANGUAGE:C>:-Xpass_source>
$<$<COMPILE_LANGUAGE:ASM>:-Xcpu=g3kh>
$<$<COMPILE_LANGUAGE:ASM>:-goptimize>
$<$<COMPILE_LANGUAGE:ASM>:-g>
$<$<COMPILE_LANGUAGE:ASM>:-Xcharacter_set=utf8>
$<$<COMPILE_LANGUAGE:C>:-Xasm_option=-Xprn_path=.>
$<$<COMPILE_LANGUAGE:C>:-Xcref=.>
$<$<COMPILE_LANGUAGE:ASM>:-Xprn_path=.>
)

add_link_options(
-optimize=symbol_delete -entry=__cstart -debug -stack
-library=v850e3v5/rhf8n.lib,v850e3v5/libmalloc.lib
-start=RESET/0,EIINTTBL.const/00000200,.const,.INIT_DSEC.const,.INIT_BSEC.const,.text,.data/00008000,.data.R,.bss,.stack.bss/FEDE8000
-rom=.data=.data.R
-change_message=warning=2300,2142 -change_message=information=1321,1110 -total_size -list -show=all
)

# The following setting selects the output type of compilation of the source.
if(NOT DEFINED EXAMPLE_ALT_OUTPUT_TYPE)
  set(EXAMPLE_ALT_OUTPUT_TYPE 0) # 0: Usual object or 2: Preprocessed source or 3: Assembly source.
endif()
if(EXAMPLE_ALT_OUTPUT_TYPE EQUAL 1)
  set_property(SOURCE ${GSG_BASE_DIR}/src/test_dep_scan_etc_c.c
  APPEND PROPERTY COMPILE_OPTIONS
  $<IF:$<STREQUAL:${CMAKE_GENERATOR},Ninja>,-P,'-P'> -otest_dep_scan_etc_c.p
  )
elseif(EXAMPLE_ALT_OUTPUT_TYPE EQUAL 2)
  set_property(SOURCE ${GSG_BASE_DIR}/src/test_dep_scan_etc_c.c
  APPEND PROPERTY COMPILE_OPTIONS
  $<IF:$<STREQUAL:${CMAKE_GENERATOR},Ninja>,-S,'-S'> -otest_dep_scan_etc_c.s
  )
endif()

##########
endmacro()
##########

#----------------------------------------
# Note: Renesas pseudo exe linker options
#----------------------------------------

# -xcopt=<XConverter options>
# The `-xcopt=` is not a native option of either Renesas Linker or Renesas XConverter.
# It is intended to be used for CMake's wrapper scripts for Renesas build tools.
# The <XConvert options> are passed not to rlink.exe but to renesas_cc_converter.exe.

#------------------------------------------------------
# Note: Renesas exe linker options' additional behavior
#------------------------------------------------------

# If `-form=` is not specified, both -form=abs and -form=s are regarded as being specified.
#
# If only -form=s is specified, -form=abs is regarded as being specified. On the other hand,
# if only -form=abs is specified, -form=s is not regarded as being specified.
#
# If only either of -form=<hex|bin> is specified, -form=abs is regarded as being specified
# but -form=s is not regarded as being specified in addition to the -form=<hex|bin>.
#
# If -form=abs and one of -form=<s|hex|bin> are specified simultaneously, Renesas linker is
# executed twice. The first execution is performed with -form=abs, the second execution is
# performed with the specified one of -form=<s|hex|bin>. In the case, the following options
# are regarded as being specified for the second execution if these options are specified.
# -byte_count=<num>
# -fix_record_length_and_align=<num>
# -record=<item>
# -end_record=<item>
# -s9
# -space[=<num|item>]
# -crc=<sub_option>
# -output=<sub_option>

# If `-library=` is specified with relative paths, both of the following folders are also searched.
# <compiler path>/lib
# <compiler path>

#----------------------------------------------------
# Note: Renesas compiler options' additional behavior
#----------------------------------------------------

# In case of other than Ninja, `-P` and `-S` cannot be used. Please quote the option
# with single quotation character as follow:
# '-S'
# '-P'

#---------------------------------------------------------------------
# Note: DebugComp, Internal and Utilities folder location of e² studio
#---------------------------------------------------------------------

# Renesas' FAQ
#
# https://en-support.renesas.com/knowledgeBase/19891761
# https://ja-support.renesas.com/knowledgeBase/19851044
